#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==============================================================================
# author          :Ghislain Vieilledent
# email           :ghislain.vieilledent@cirad.fr, ghislainv@gmail.com
# web             :https://ecology.ghislainv.fr
# python_version  :>=2.7
# license         :GPLv3
# ==============================================================================

# Import
import os.path
import pywdpa


# test_get_wdpa
def test_get_wdpa():
  #pywdpa.get_wdpa("MDG")
  #f = os.path.isfile("pa_MDG.shp")
  #assert f is True
  f = "File"
  assert f == "File"

# End



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==============================================================================
# author          :Ghislain Vieilledent
# email           :ghislain.vieilledent@cirad.fr, ghislainv@gmail.com
# web             :https://ecology.ghislainv.fr
# python_version  :>=2.7
# license         :GPLv3
# ==============================================================================

"""pywdpa.pywdpa: provides entry point main()."""

__version__ = "0.1.1"

import sys
from .get_wdpa import get_wdpa

def main():
    print("Executing pywdpa version %s." % __version__)
    print("For country with isocode: %s" % sys.argv[1])
    get_wdpa(sys.argv[1])
    return None

# End



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==============================================================================
# author          :Ghislain Vieilledent
# email           :ghislain.vieilledent@cirad.fr, ghislainv@gmail.com
# web             :https://ecology.ghislainv.fr
# python_version  :>=2.7
# license         :GPLv3
# ==============================================================================

from __future__ import division, print_function  # Python 3 compatibility
from .get_token import get_token
from .get_wdpa import get_wdpa

# End



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==============================================================================
# author          :Ghislain Vieilledent
# email           :ghislain.vieilledent@cirad.fr, ghislainv@gmail.com
# web             :https://ecology.ghislainv.fr
# python_version  :>=2.7
# license         :GPLv3
# ==============================================================================

# Import
from __future__ import division, print_function  # Python 3 compatibility
import os
from osgeo import ogr
import requests
import json
from .get_token import get_token
import numpy as np


# get_wdpa()
def get_wdpa(iso3):

    """This function downloads protected areas for one country using the
    WDPA API.

    Protected areas defined by a point are not considered.

    :param iso3: The ISO-3 code of the country.

    :return: None. The shapefile is written on the hard drive
    (in the current directory).

    """

    os.environ["SHAPE_ENCODING"] = "utf-8"
    
    base_url = "https://api.protectedplanet.net/"
    category = "v3/countries/"
    wdpa_token = get_token()
    request = base_url + category + iso3 + "?token=" + wdpa_token
    response = requests.get(request)

    if (response.status_code == 404):
        return "Invalid ISO-3 code"

    response = json.loads(response.text) # Equivalent to response.json()

    pas_count = response["country"]["pas_count"]
    pages = list(range(np.int(np.ceil(pas_count / 50.0))))

    if (pas_count is not None):

        # Create the output shapefile
        output_file = "pa_" + iso3 + ".shp" 
        driver = ogr.GetDriverByName("ESRI Shapefile")
        if os.path.exists(output_file):
            driver.DeleteDataSource(output_file)
        ds = driver.CreateDataSource(output_file)
        # Create the spatial reference, WGS84
        srs = ogr.osr.SpatialReference()
        srs.ImportFromEPSG(4326)
        layer = ds.CreateLayer("wdpa", srs, ogr.wkbMultiPolygon)
        # Add attributes
        layer.CreateField(ogr.FieldDefn("wdpa_id", ogr.OFTInteger))
        layer.CreateField(ogr.FieldDefn("pa_name", ogr.OFTString))
        layer.CreateField(ogr.FieldDefn("ctry_iso3", ogr.OFTString))
        layer.CreateField(ogr.FieldDefn("is_marine", ogr.OFTString))
        layer.CreateField(ogr.FieldDefn("type", ogr.OFTString))
        layer.CreateField(ogr.FieldDefn("iucn_cat", ogr.OFTString))

        # API
        category = "v3/protected_areas/search/"

        # Loop on pages
        for p in pages:
            request = "".join(
                [base_url, category, "?token=", wdpa_token,
                 "&with_geometry=", "true", "&country=",
                 iso3, "&per_page=", str(50), "&page=", str(p+1)]
            )

            response = requests.get(request)
            response = response.json()
            response = response["protected_areas"]

            # Number of protected areas
            npa = len(response)

            # Loop on protected areas
            for i in range(npa):

                # Get pa geometry from json
                pa = response[i]
                g = ogr.CreateGeometryFromJson(
                    json.dumps(pa["geojson"]["geometry"])
                )

                # # If point, convert to polygon with a buffer
                # if (g.GetGeometryName() is "POINT" and pa["reported_area"] is not "0.0"):
                #     area = np.float(pa["reported_area"])
                #     buffer_size = int(round(np.sqrt(area/np.pi)*1000))
                #     g = g.Buffer(buffer_size)

                # If polygon, add feature
                if (g.GetGeometryName() == "POLYGON"):
                    # Create feature with geometry and attributes
                    featureDefn = layer.GetLayerDefn()
                    feature = ogr.Feature(featureDefn)
                    feature.SetGeometry(g)
                    feature.SetField("wdpa_id", pa["wdpa_id"])
                    feature.SetField("pa_name", pa["name"])
                    feature.SetField("ctry_iso3", iso3)
                    feature.SetField("is_marine", str(pa["marine"]))
                    feature.SetField("type", pa["designation"]["name"])
                    feature.SetField("iucn_cat", pa["iucn_category"]["name"])
                    # Add feature to layer
                    layer.CreateFeature(feature)
                    # Dereference the feature
                    feature = None

        # Save and close the data source
        ds = None
        return None

    else:
        return ("The WDPA does not contain any protected areas for " + iso3)

# End



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==============================================================================
# author          :Ghislain Vieilledent
# email           :ghislain.vieilledent@cirad.fr, ghislainv@gmail.com
# web             :https://ecology.ghislainv.fr
# python_version  :>=2.7
# license         :GPLv3
# ==============================================================================

# Import
import pywdpa


# test_get_token
def test_get_token():
  token = pywdpa.get_token(key="")
  r = ["\nMissing WDPA API token. Please ensure that:\n",
       "1) You completed this form [https://api.protectedplanet.net/request] to get the token,\n",
       "2) You stored the value as an environment variable with the recommended name WDPA_KEY."]
  assert token == "".join(r)

# End



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==============================================================================
# author          :Ghislain Vieilledent
# email           :ghislain.vieilledent@cirad.fr, ghislainv@gmail.com
# web             :https://ecology.ghislainv.fr
# python_version  :>=2.7
# license         :GPLv3
# ==============================================================================

# Import
from __future__ import division, print_function  # Python 3 compatibility
import os
import requests


# get_token()
def get_token(key="WDPA_KEY"):

    """Check Protected Planet API token.

    This function checks if the user has stored a valid Protected
    Planet API token as an environment variable file under the
    key \code{WDPA_KEY}.

    Before using this package for the first time, the user must follow
    these steps: 

    1. Fill in the form available at
    <https://api.protectedplanet.net/request> to obtain a personal API
    token.

    2. Store the token as an environment variable under the key
    "WDPA_KEY". You can use the command
    os.environ["WDPA_KEY"]="your_token" or python-dotenv:
    <https://github.com/theskumar/python-dotenv>.

    :param key: Environment variable name (recommended name: "WDPA_KEY").

    :return: A vector of length one with the value of the API token.

    """

    wdpa_key = os.getenv(key)

    if wdpa_key is None:
        return ("\nMissing WDPA API token. Please ensure that:\n" +
                "1) You completed this form [https://api.protectedplanet.net/request] to get the token,\n" +
                "2) You stored the value as an environment variable with the recommended name WDPA_KEY.")

    response = requests.get("https://api.protectedplanet.net/test?token=" + wdpa_key)

    if (response.status_code == 401):
        return ("\nInvalid WDPA API token. Please ensure that:\n" +
                "1) You completed this form [https://api.protectedplanet.net/request] to get the token,\n" +
                "2) You stored the value as an environment variable with the recommended name WDPA_KEY.")

    if (response.status_code != 200):
        return "\nSomething goes wrong with your API token."

    return(wdpa_key)

# End



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==============================================================================
# author          :Ghislain Vieilledent
# email           :ghislain.vieilledent@cirad.fr, ghislainv@gmail.com
# web             :https://ecology.ghislainv.fr
# python_version  :>=2.7
# license         :GPLv3
# ==============================================================================

"""Convenience wrapper for running pywdpa directly from source tree.
See https://gehrcke.de/2014/02/distributing-a-python-command-line-application/
"""

from pywdpa.pywdpa import main

if __name__ == '__main__':
    main()

# End



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==============================================================================
# author          :Ghislain Vieilledent
# email           :ghislain.vieilledent@cirad.fr, ghislainv@gmail.com
# web             :https://ecology.ghislainv.fr
# python_version  :>=2.7
# license         :GPLv3
# ==============================================================================

# Import
import re # Regular expression
from setuptools import setup

# Version
version = re.search(
    '^__version__\s*=\s*"(.*)"',
    open('pywdpa/pywdpa.py').read(),
    re.M
    ).group(1)

# Markdown README file
with open("README.md", "rb") as f:
    long_description = f.read().decode("utf-8")

# Setup
setup(name="pywdpa",
      version=version,
      author="Ghislain Vieilledent",
      author_email="ghislain.vieilledent@cirad.fr",
      url="https://github.com/ghislainv/pywdpa",
      license="GPLv3",
      description="This is the Python 'pywdpa' package",
      long_description=long_description,
      long_description_content_type="text/markdown",
      classifiers=["Development Status :: 4 - Beta",
                   "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
                   "Programming Language :: Python :: 2",
                   "Programming Language :: Python :: 3",
                   "Operating System :: OS Independent",
                   "Topic :: Scientific/Engineering :: Bio-Informatics"],
      keywords="protected areas wdpa world database on protected areas",
      python_requires=">=2.7",
      packages=["pywdpa"],
      package_dir={"pywdpa": "pywdpa"},
      entry_points = {"console_scripts": ["pywdpa = pywdpa.pywdpa:main"]},
      install_requires=["numpy", "gdal", "requests"],
      zip_safe=False)

# End



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ==============================================================================
# author          :Ghislain Vieilledent
# email           :ghislain.vieilledent@cirad.fr, ghislainv@gmail.com
# web             :https://ecology.ghislainv.fr
# python_version  :>=2.7
# license         :GPLv3
# ==============================================================================

"""pywdpa.__main__: executed when pywdpa is called as script."""

# Import
from .pywdpa import main

# Execute main()
main()

# End
